/*
	NGR Converter - National Grid Reference to Latitude and Longitude
	Copyright © 2005-2009 Harry Whitfield

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin St, Fifth Floor, Boston, MA	 02110-1301	 USA
	
	NGR Converter - version 1.5
	22 May, 2009
	Copyright © 2005-2009 Harry Whitfield
	mailto:g6auc@arrl.net
*/

function gridRefOfLetterPair(pair)
{
	pair = pair.toUpperCase();
	
	var error = new GridRef(NaN, NaN);
	
	var first = "HNSOT";
	var E1 = [0, 0, 0, 500000, 500000];
	var N1 = [1000000, 500000, 0, 500000, 0];
	
	var idx = first.indexOf(pair[0]);
	if (idx < 0) return error;
	var eastings  = E1[idx];
	var northings = N1[idx];
	
	var second = "VWXYZQRSTULMNOPFGHJKABCDE";
	var valid  = [ 	"PTUYZ",
					"ABCDFGHJKLMNORSTUWXYZ",
					"CDEGHJKMNOPRSTUVWXYZ",
					"V",
					"AFGLMQRV" ];
					
	var idx2 = valid[idx].indexOf(pair[1]);
	if (idx2 < 0) return error;
	
	idx = second.indexOf(pair[1]);
	if (idx < 0) return error;		// should not happen!
	
	var x = idx % 5;
	eastings += 100000*x;
	
	var y = Math.round( (idx - x) / 5);
	northings += 100000*y;
	
	//print('gridRefOfLetterPair:' + eastings + ',' + northings);
	
	return new GridRef(eastings, northings);
}

function letterPairOfGridRef(gridRef)
{
	var eastings  = Math.floor(gridRef.eastings );
	var northings = Math.floor(gridRef.northings);
	
	var idx = Math.floor(eastings  / 500000);
	var idy = Math.floor(northings / 500000);
	
	//print('1:letterPairOfGridRef:' + idx + ',' + idy);
	
	if ((idx < 0) || (idx > 1)) return "";
	if ((idy < 0) || (idx > 2)) return "";
	
	var first = [ ["S", "N", "H"], ["T", "O", "J"] ];
	var pair = first[idx][idy];
	
	eastings  -= 500000*idx;
	northings -= 500000*idy;
	
	idx = Math.floor(eastings  / 100000);
	idy = Math.floor(northings / 100000);
	
	//print('2:letterPairOfGridRef:' + idx + ',' + idy);

	var second = "VWXYZQRSTULMNOPFGHJKABCDE";

	pair += second[5*idy + idx];
	//print('pair=' + pair);
	
	var G = gridRefOfLetterPair(pair);
	//print('G.eastings=' + G.eastings);
	
	if (isNaN(G.eastings)) return "";
	return pair;
}

function isRef(data)
{
	data = data.toUpperCase();
	
	var first = "HNSOT";
	var second = "VWXYZQRSTULMNOPFGHJKABCDE";
	var digit = "0123456789";
	
	var idx = first.indexOf(data[0]);
	if (idx < 0) return false;
	idx = second.indexOf(data[1]);
	if (idx < 0) return false;
	if ( (data.length % 2) != 0 ) return false;
	if ( (data.length < 4) || (data.length > 12) )  return false;
	
	for (var i = 2; i < data.length; i++)
	{
		idx = digit.indexOf(data[i]);
		if (idx < 0) return false;
	}
	return true;
}

function get_ref(data)	// input grid reference
{
	if (isRef(data))
	{
		var pair = data.substring(0, 2);
		var rest = data.substring(2);
		var len  = rest.length >> 1;
		var eastings  = rest.substring(0, len);
		var northings = rest.substring(len);
		
		while (eastings.length < 5 ) eastings  += "0";
		while (northings.length < 5) northings += "0";
		
		eastings  = Number(eastings);
		northings = Number(northings);
		
		var G = gridRefOfLetterPair(pair);
		
		eastings  += G.eastings;
		northings += G.northings;
		
		return new GridRef(eastings, northings);
	}

	var item = data.split(',', 2);	// 400000, 100000

	var e = Number(item[0]); if (isNaN(e)) return new GridRef(NaN, NaN);
	if ((e < 0) || (e >= 700000))          return new GridRef(NaN, NaN);

	var n = Number(item[1]); if (isNaN(n)) return new GridRef(NaN, NaN);
	if ((n < 0) || (e >= 1300000))         return new GridRef(NaN, NaN);

	return new GridRef(e, n);
}

function put_ref(G)
{
  	var eastings  = G.eastings;
  	var northings = G.northings;

  	if (numericRefs) return eastings.toFixed(prec2) + "," + northings.toFixed(prec2);

	eastings  = (Math.floor(eastings)  % 100000).toString();
	northings = (Math.floor(northings) % 100000).toString();
	
	while (eastings.length  < 5) eastings  = "0" + eastings;
	while (northings.length < 5) northings = "0" + northings;
	
	var figs = figuresPref >> 1;
	
	eastings  = eastings.substring(0, figs);
	northings = northings.substring(0, figs);
	
	var pair = letterPairOfGridRef(G);

	if (pair != "") return pair + eastings + northings;
	return "Invalid Reference";
}

function get_ang(data, dmax, NE, SW) {	// input angle
	var item, a, d, m, s, t;
	
	//print("get_ang: " + data + ', ' + dmax + ', ' + NE + ', ' + SW);
	
	item = data.split(' ', 4);		// 49 45 58.2689 N   or  7 33 23.2145 W
	//print("item: " + item);
	
	if (item.length < 2) { return NaN; }

	a = item[item.length - 1];
	a = a.toUpperCase();
	if ((a != NE) && (a != SW)) { return NaN; }

	d = Number(item[0]); if (isNaN(d)) { return NaN; }
	if ((d < 0) || (d >= dmax)) { return NaN; }
	if (item.length == 2) {
		m = 0;
		s = 0;
	} else {
		if (d != Math.floor(d)) { return NaN; }
		m = Number(item[1]); if (isNaN(m)) { return NaN; }
		if (( m < 0) || (m >= 60)) { return NaN; }
		if (item.length == 3) {
			s = 0;
		} else {
			if (m != Math.floor(m)) { return NaN; }
			s = Number(item[2]); if (isNaN(s)) { return NaN; }
			if ( ( s < 0 ) || ( s >= 60 ) ) { return NaN; }
		}
	}
	
	t = (d + m / 60.0 + s / 3600.0) * pi / 180.0;
	if ( a == SW ) { t = -t; }
	
	//print(t * 180 / pi);

	return t;
}

function Xget_ang(data, dmax, NE, SW)	// input angle
{
	var item = data.split(' ', 4);	// 49 45 58.2689 N   or  7 33 23.2145 W

	var d = Number(item[0]); if (isNaN(d)) return NaN;
	var m = Number(item[1]); if (isNaN(m)) return NaN;
	var s = Number(item[2]); if (isNaN(s)) return NaN;
	var a = item[3]; if (a == undefined)   return NaN;

	if ( ( d < 0 ) || ( d >= dmax ) || ( d != Math.floor(d) ) ) return NaN;
	if ( ( m < 0 ) || ( m >= 60   ) || ( m != Math.floor(m) ) ) return NaN;
	if ( ( s < 0 ) || ( s >= 60 ) ) return NaN;

	a = a.toUpperCase();
	if (  (a != NE) && ( a != SW) ) return NaN;

	var t = (d + m / 60.0 + s / 3600.0) * pi / 180.0;
	if ( a == SW ) t = -t;

	return t;
}

function put_ang(St, K) 						// output angle
{
	var d, m;
	var s;
	var sign;

	sign = '';
	if ( K < 0 )
	{
		K = -K;
		sign = '-';
	}
	K = 180.0 * K / pi;
	d = Math.floor(K);
	K = 60.0 * (K - d);
	m = Math.floor(K);
	s = 60.0 * (K - m);
	return St + sign + d + ' ' + m + ' ' + s.toFixed(prec3);
}

function put_lat(K)									// output angle 
{
	var d, m;
	var s;
	var ns;

	ns = 'N';
	if ( K < 0 )
	{
		K = -K;
		ns = 'S';
	}
	K = 180.0 * K / pi;
	d = Math.floor(K);
	K = 60.0 * (K - d);
	m = Math.floor(K);
	s = 60.0 * (K - m);
	return d + ' ' + m + ' ' + s.toFixed(prec1) + ' ' + ns;
}

function put_long(K)								// output angle 
{
	var d, m;
	var s;
	var ew;

	ew = 'E';
	if ( K < 0 )
	{
		K = -K;
		ew = 'W';
	}
	K = 180.0 * K / pi;
	d = Math.floor(K);
	K = 60.0 * (K - d);
	m = Math.floor(K);
	s = 60.0 * (K - m);
	return d + ' ' + m + ' ' + s.toFixed(prec1) + ' ' + ew;
}

function put_convergence(K, L)
{
	return put_ang("", FNC(K, L));
}

function put_scale(K, L)
{
	return FNF(K, L);
}
