/*
	NGR Converter - National Grid Reference to Latitude and Longitude
	Copyright © 2005-2009 Harry Whitfield

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin St, Fifth Floor, Boston, MA	 02110-1301	 USA
	
	NGR Converter - version 1.5
	22 May, 2009
	Copyright © 2005-2009 Harry Whitfield
	mailto:g6auc@arrl.net
*/

var LB = "AA00AA00";
var UB = "RR99XX99";

function get_locator(S)	// return latitude and longitude given locator
{
	if (S.length == 6) S += '55';
	if (S.length != 8) return new Position(NaN, NaN);

	var Loc = new Array();

	for (var  i = 0; i < 8; i++ )
	{
		var c = S[i]; c = c.toUpperCase();
		if  ( ( LB[i] <= c) && ( c <= UB[i] ) )
		{
			Loc[i] = c.charCodeAt(0) - LB[i].charCodeAt(0);
		}
		else return new Position(NaN, NaN);
	}

	var KK = (10.0 * Loc[1] + Loc[3] - 90.0 + (100.0 * Loc[5] + 10.0 * Loc[7] + 5.0) / 2400.0) * pi / 180.0;
	var LL = (10.0 * Loc[0] + Loc[2] - 90.0 + (100.0 * Loc[4] + 10.0 * Loc[6] + 5.0) / 2400.0) * pi /  90.0;

	return new Position(KK, LL);
}

function put_locator (K, L)	// return locator given latitude and longitude
{
	var d, e;
	var Loc = new Array();

	d = 180.0 * K / pi + 90.0;
	Loc[1] = Math.floor(d / 10.0);
	d = d - 10.0 * Loc[1];
	Loc[3] = Math.floor(d);
	d = d - Loc[3];
	Loc[5] = Math.floor(24.0 * d);
	d = 24.0 * d - Loc[5];
	Loc[7] = Math.floor(10.0 * d);

	e = 90.0 * L / pi + 90.0;
	Loc[0] = Math.floor(e / 10.0);
	e = e - 10.0 * Loc[0];
	Loc[2] = Math.floor(e);
	e = e - Loc[2];
	Loc[4] = Math.floor(24.0 * e);
	e = 24.0 * e - Loc[4];
    Loc[6] = Math.floor(10.0 * e);

	var s = new Array();
	for (var i = 0; i < 8; i++ )
	{
		s[i] = String.fromCharCode( Loc[i] + LB[i].charCodeAt(0) );
	}
	
  	if (s[1] == 'S')
  	{
    	s[1] = 'R';
    	s[3] = '9';
    	s[5] = 'X';
    	s[7] = '9';
	}
  	if (s[0] == 'S') s[0] = 'A';
  	
  	var res = '';
	for (var i = 0; i < 8; i++ ) res += s[i];
	return res;
}
