/*
	NGR Converter - National Grid Reference to Latitude and Longitude
	Copyright © 2005-2009 Harry Whitfield

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin St, Fifth Floor, Boston, MA	 02110-1301	 USA
	
	NGR Converter - version 1.5
	22 May, 2009
	Copyright © 2005-2009 Harry Whitfield
	mailto:g6auc@arrl.net
*/

var prec1 = 4;					// precision for seconds of arc
var prec2 = 3;					// precision for eastings, northings
var prec3 = 8;					// precision for convergence

var numericRefs = (preferences.numericRefs.value == "1");
var figuresPref =  preferences.figuresPref.value;
var locatorPref =  preferences.locatorPref.value;

include("math.js");
include("locator.js");
include("io.js");

var E = 400000;
var N = -100000;
var G = new GridRef(E, N);

var K = 49.0;
var L = -2.0;
var P = new Position(K, L);

var item = new Array();

var minWidth = 275;
var midWidth = 300;
var maxWidth = 325;

var curWidth  = minWidth;
if (numericRefs) curWidth  = midWidth;
var userWidth = curWidth;

var debug = false;

var redButtonWithin = false;
var amberButtonWithin = false;
var greenButtonWithin = false;

var hasFocus = false;

function setAmber(full)
{
	if (full)
	{
		amberButton.src = "Resources/buttons/amberMinus.png";
	}
	else
	{
		amberButton.src = "Resources/buttons/amber.png";
	}
}

function normalButtons()
{
	redButton.src   = "Resources/buttons/red.png";
	greenButton.src = "Resources/buttons/green.png";
	setAmber(numericRefs);
}

function activeButtons()
{
	redButton.src   = "Resources/buttons/redCross.png";
	greenButton.src = "Resources/buttons/greenPlus.png";
	setAmber(numericRefs);
}

function focusButtons()
{
	if (hasFocus)
	{
		redButton.src   = "Resources/buttons/red.png";
		greenButton.src = "Resources/buttons/green.png";
		setAmber(numericRefs);
	}
	else
	{
		redButton.src   = "Resources/buttons/grey.png";
		amberButton.src = "Resources/buttons/grey.png";
		greenButton.src = "Resources/buttons/grey.png";
	}
}

function setTooltips(full)
{
	if (full)
	{
		amberButton.tooltip = "Click to display alphanumeric references.";
	}
	else
	{
		amberButton.tooltip = "Click to display numeric references.";
	}
}

function toggleMode()
{
	numericRefs = ! numericRefs;
	setAmber(numericRefs);
	setTooltips(numericRefs);
	if (numericRefs)
	{
		preferences.numericRefs.value = "1";
	}
	else
	{
		preferences.numericRefs.value = "0";
	}
	savePreferences();
	
	if (reference.data  != "")
	{
		var G = get_ref(reference.data);
		if ( isNaN(G.eastings) )
		{
			reference.data = 'Invalid Input';
		}
		else reference.data = put_ref(G);
	}
}

function toggleSize()
{
	if (main_window.width == userWidth)
	{
		if (numericRefs) { setUp(midWidth); } else setUp(minWidth);
	}
	else
	{
		setUp(userWidth);
	}
}

function setUp(width) {
	if (debug) {
		print('setUp():======== start ========:');
		print('width=' + width);
		print('minWidth=' + minWidth);
		print('maxWidth=' + maxWidth);
	}
	
	if (width < minWidth) {
		width = minWidth;
	}
	if (width > maxWidth) {
		width = maxWidth;
	}

	main_window.width = width;
	middleTop.width  =
	middleMid.width  =
	middleLow.width  = width - 80;
	
	rightTop.hOffset =
	rightMid.hOffset =
	rightLow.hOffset = width - 40;
	
	reference.width =
	latitude.width =
	longitude.width =
	locator.width = width - 80;
	
	dragbox.hOffset = width - 42;

	midBgVoltage.width =
	midBgCurrent.width =
	midBgFactor.width  =
	midBgPower.width   = width - 92;

	rightBgVoltage.hOffset =
	rightBgCurrent.hOffset =
	rightBgFactor.hOffset  =
	rightBgPower.hOffset   = width - 46;

	updateNow();
	
	return width;
}

function updatePrefs()
{
	reference.font = latitude.font = longitude.font = locator.font =  preferences.generalFontPref.value;
	userWidth = curWidth;
	setUp(curWidth);
	
	numericRefs = (preferences.numericRefs.value == "1");
	figuresPref =  preferences.figuresPref.value;
	locatorPref =  preferences.locatorPref.value;
	locator.editable = (locatorPref == "IARU Locator");
	
	if (locatorPref == "IARU Locator") locator.tooltip = "Enter a 6 or 8 character IARU locator in this box.";
	if (locatorPref == "Convergence")  locator.tooltip = "Computed Convergence.";
	if (locatorPref == "Scale")        locator.tooltip = "Computed Scale.";

	if (reference.data  != "")
	{
		var G = get_ref(reference.data);
		if ( isNaN(G.eastings) )
		{
			reference.data = 'Invalid Input';
		}
		else reference.data = put_ref(G);
	}
	
	setAmber(numericRefs);
	setTooltips(numericRefs);
}

function test_case(G)
{
 			var P = lat_long(G.eastings, G.northings);
   			reference.data = put_ref(G);
	latitude.data  = put_lat(P.latitude);
	longitude.data = put_long(P.longitude);
	if (locatorPref == "IARU Locator") locator.data = put_locator(P.latitude, P.longitude);
	if (locatorPref == "Convergence")  locator.data = put_convergence(P.latitude, P.longitude);
	if (locatorPref == "Scale")        locator.data = put_scale(P.latitude, P.longitude);
}

dragbox.onMouseDrag = function () {
	userWidth = setUp(dragbox.hOffset + system.event.x + 32);
};

redButton.onMouseUp    = function () {
	if (redButtonWithin) {
		normalButtons();
		closeWidget();
	}
};
redButton.onMouseDown  = function () {
	redButtonWithin = true;
	activeButtons();
};

redButton.onMouseEnter = function () {
	redButtonWithin = true;
	activeButtons();
};
redButton.onMouseExit  = function () {
	redButtonWithin = false;
	focusButtons();
};

amberButton.onMouseUp    = function () {
	testLicense();
	if (amberButtonWithin) {
		normalButtons();
		toggleMode();
	} 
};
amberButton.onMouseDown  = function () {
	testLicense(); 
	amberButtonWithin = true; 
	activeButtons(); 
};
amberButton.onMouseEnter = function () {
	amberButtonWithin = true;
	activeButtons();
};
amberButton.onMouseExit  = function () {
	amberButtonWithin = false;
	focusButtons();
};

greenButton.onMouseUp    = function () {
	testLicense();
	if (greenButtonWithin) {
		normalButtons();
		toggleSize();
	}
};
greenButton.onMouseDown  = function () {
	testLicense();
	greenButtonWithin = true;
	activeButtons();
};
greenButton.onMouseEnter = function () {
	greenButtonWithin = true;
	activeButtons();
};
greenButton.onMouseExit  = function () {
	greenButtonWithin = false;
	focusButtons();
};

reference.onKeyPress = function () {
			testLicense();
			if (system.event.keyString == "Tab")
			{
				reference.rejectKeyPress();
				reference.select(-1,-1);
				latitude.focus();
				latitude.select(0, -1);
			}
			else if ((system.event.keyString == "Return") || (system.event.keyString == "Enter"))
			{
				reference.rejectKeyPress();
				
				if (reference.data !== "")
				{
					G = get_ref(reference.data);
					if (isNaN(G.eastings))
					{
						latitude.data  = 'Invalid Input';
						longitude.data = '';
						locator.data   = '';
					}
					else
					{
						P = lat_long(G.eastings, G.northings);
						latitude.data  = put_lat(P.latitude);
						longitude.data = put_long(P.longitude);
						if (locatorPref == "IARU Locator") { locator.data = put_locator(P.latitude, P.longitude); }
						if (locatorPref == "Convergence")  { locator.data = put_convergence(P.latitude, P.longitude); }
						if (locatorPref == "Scale")		   { locator.data = put_scale(P.latitude, P.longitude); }
					}
				}
			}
			else
			{
				latitude.data = "";
				longitude.data = "";
				locator.data = "";
			}
};
reference.onGainFocus = function () { 
	leftBgVoltage.src  = "Resources/boxes/tbhlleft.png";
	midBgVoltage.src   = "Resources/boxes/tbhlmiddle.png";
	rightBgVoltage.src = "Resources/boxes/tbhlright.png";
};
reference.onLoseFocus = function () { 
	leftBgVoltage.src  = "Resources/boxes/tbleft.png";
	midBgVoltage.src   = "Resources/boxes/tbmiddle.png";
	rightBgVoltage.src = "Resources/boxes/tbright.png";
};

latitude.onKeyPress = function () {
			testLicense();
			if (system.event.keyString == "Tab")
			{
				latitude.rejectKeyPress();
				latitude.select(-1,-1);
				longitude.focus();
				longitude.select(0, -1);
			}
			else if ((system.event.keyString == "Return") || (system.event.keyString == "Enter"))
			{
				latitude.rejectKeyPress();
				
				if (latitude.data !== "")
				{
					K = get_ang(latitude.data,	 90, 'N', 'S');
					L = get_ang(longitude.data, 180, 'E', 'W');
				
					if (isNaN(K) || isNaN(L))
					{
						reference.data = 'Invalid Input';
						locator.data   = '';
					}
					else
					{
						G = ngr(K, L);
						reference.data = put_ref(G);
						if (locatorPref == "IARU Locator") { locator.data = put_locator(K, L); }
						if (locatorPref == "Convergence")  { locator.data = put_convergence(K, L); }
						if (locatorPref == "Scale")		   { locator.data = put_scale(K, L); }
					}
				}
			}
			else
			{
				reference.data = "";
				locator.data = "";
			}
};
latitude.onGainFocus = function () { 
	leftBgCurrent.src  = "Resources/boxes/tbhlleft.png";
	midBgCurrent.src   = "Resources/boxes/tbhlmiddle.png";
	rightBgCurrent.src = "Resources/boxes/tbhlright.png";
};
latitude.onLoseFocus = function () { 
	leftBgCurrent.src  = "Resources/boxes/tbleft.png";
	midBgCurrent.src   = "Resources/boxes/tbmiddle.png";
	rightBgCurrent.src = "Resources/boxes/tbright.png";
};	

longitude.onKeyPress = function () {
			testLicense();
			if (system.event.keyString == "Tab")
			{
				longitude.rejectKeyPress();
				longitude.select(-1,-1);
				locator.focus();
				locator.select(0, -1);
			}
			else if ((system.event.keyString == "Return") || (system.event.keyString == "Enter"))
			{
				longitude.rejectKeyPress();
				
				if (latitude.data !== "")
				{
					K = get_ang(latitude.data,	 90, 'N', 'S');
					L = get_ang(longitude.data, 180, 'E', 'W');
				
					if (isNaN(K) || isNaN(L))
					{
						reference.data = 'Invalid Input';
						locator.data   = '';
					}
					else
					{
						G = ngr(K, L);
						reference.data = put_ref(G);
						if (locatorPref == "IARU Locator") { locator.data = put_locator(K, L); }
						if (locatorPref == "Convergence")  { locator.data = put_convergence(K, L); }
						if (locatorPref == "Scale")		   { locator.data = put_scale(K, L); }
					}
				}
			}
			else
			{
				reference.data = "";
				locator.data = "";
			}
};
longitude.onGainFocus = function () { 
	leftBgFactor.src  = "Resources/boxes/tbhlleft.png";
	midBgFactor.src   = "Resources/boxes/tbhlmiddle.png";
	rightBgFactor.src = "Resources/boxes/tbhlright.png";
};
longitude.onLoseFocus = function () { 
	leftBgFactor.src  = "Resources/boxes/tbleft.png";
	midBgFactor.src   = "Resources/boxes/tbmiddle.png";
	rightBgFactor.src = "Resources/boxes/tbright.png";
};

locator.onKeyPress = function () {
			testLicense();
			if (system.event.keyString == "Tab")
			{
				locator.rejectKeyPress();
				locator.select(-1,-1);
				reference.focus();
				reference.select(0, -1);
			}
			else if ((system.event.keyString == "Return") || (system.event.keyString == "Enter"))
			{
				locator.rejectKeyPress();
				
				if (locator.data !== "")
				{
					P = get_locator(locator.data);
					if (isNaN(P.latitude))
					{
						reference.data = 'Invalid Input';
						latitude.data	= '';
						longitude.data	= '';
					}
					else
					{
						latitude.data  = put_lat(P.latitude);
						longitude.data = put_long(P.longitude);
				
						G = ngr(P.latitude, P.longitude);
						reference.data = put_ref(G);
					}
				}
			}
			else
			{
				reference.data = "";
				latitude.data = "";
				longitude.data = "";
			}
};
locator.onGainFocus = function () { 
	leftBgPower.src  = "Resources/boxes/tbhlleft.png";
	midBgPower.src   = "Resources/boxes/tbhlmiddle.png";
	rightBgPower.src = "Resources/boxes/tbhlright.png";
};
locator.onLoseFocus = function () { 
	leftBgPower.src  = "Resources/boxes/tbleft.png";
	midBgPower.src   = "Resources/boxes/tbmiddle.png";
	rightBgPower.src = "Resources/boxes/tbright.png";
};

updatePrefs();
test_case(new GridRef(651409.903,313177.270));
focusWidget();
