﻿/*
	NGR Converter - National Grid Reference to Latitude and Longitude
	Copyright © 2005-2009 Harry Whitfield

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin St, Fifth Floor, Boston, MA	 02110-1301	 USA
	
	NGR Converter - version 1.5
	22 May, 2009
	Copyright © 2005-2009 Harry Whitfield
	mailto:g6auc@arrl.net
	
	Based on an Ordnance Survey Information Booklet, March 1983.
*/

var n  = 0.001673220250;
var e2 = 0.006670539762;
var a  = 6375020.481;
var b  = 6353722.490;
var F0 = 0.9996012717;

var Eorg =  400000.0;
var Norg = -100000.0;

var pi = Math.PI;

var n2 = n * n;
var n3 = n * n * n;

var Korg = 49.0 * pi / 180.0;
var Lorg = -2.0 * pi / 180.0;

function GridRef(eastings, northings)
{
	this.eastings  = eastings;
	this.northings = northings;
}

function Position(latitude, longitude)
{
	this.latitude  = latitude;
	this.longitude = longitude;
}

function FNM(k3, k4)
{
	var j3 = (1 + n + 1.25 * n2 + 1.25 * n3) * k3;
	var j4 = (3 * n + 3 * n2 + 2.625 * n3) * Math.sin(k3) * Math.cos(k4);
	var j5 = (1.875 * n2 + 1.875 * n3) * Math.sin(2 * k3) * Math.cos(2 * k4);
	var j6 = (35.0 / 24.0) * n3 * Math.sin(3 * k3) * Math.cos(3 * k4);
	return ( b * (j3 - j4 + j5 - j6) );
}

function FNV(K)
{
	var s = Math.sin(K);
	return ( a / Math.sqrt(1 - e2 * s * s) );
}

function FNR(V, K)
{
	var s = Math.sin(K);
	return ( V * (1 - e2) / (1 - e2 * s * s) );
}

function ngr(K, L) 	// lat,long -> eastings,northings
{
	var M = FNM(K - Korg, K + Korg);
	var V = FNV(K);
	var R = FNR(V, K);
	var H2 = V / R - 1.0;
	var s = Math.sin(K);
	var c = Math.cos(K);
	var t = Math.tan(K);
	var t2 = t * t;
	var t4 = t2 * t2;
	var c2 = c * c;
	var c3 = c * c2;
	var c5 = c2 * c3;
	var P = L - Lorg;
	var j3 = M + Norg;
	var j4 = 0.5 * V * s * c;
	var j5 = (V / 24) * s * c3 * (5 - t2 + 9 * H2);
	var j6 = (V / 720) * s * c5 * (61 - 58 * t2 + t4);
	var N = j3 + P * P * j4 + P * P * P * P * j5 + P * P * P * P * P * P * j6;
	var j7 = V * c;
	var j8 = (V / 6) * c3 * (V / R - t2);
	var j9 = (V / 120) * c5 * (5 - 18 * t2 + t4 + 14 * H2 - 58 * t2 * H2);
	var E = Eorg + P * j7 + P * P * P * j8 + P * P * P * P * P * j9;
	return new GridRef(E, N);
}

function Phi(N)
{
	var K = Korg;
	var M = 0;
	do
	{
		K = K + (N - Norg - M) / a;
		M = FNM(K - Korg, K + Korg);
	} while ( ! ( Math.abs(N - Norg - M) < 0.001 ) );
	return K;
}

function lat_long(E, N)	// ngr -> latitude,longitude
{
	var K = Phi(N);
	var c = Math.cos(K);
	var t = Math.tan(K);
	var t2 = t * t;
	var t4 = t2 * t2;
	var t6 = t2 * t4;
	var V = FNV(K);
	var R = FNR(V, K);
	var H2 = V / R - 1.0;
	var y = E - Eorg;
	var V = 0.000001 * V;
	var R = 0.000001 * R;
	var y = 0.000001 * y;
	var j3 = t / (2 * R * V);
	var j4 = t / (24 * R * V * V * V) * (5 + 3 * t2 + H2 - 9 * t2 * H2);
	var j5 = t / (720 * R * V * V * V * V * V) * (61 + 90 * t2 + 45 * t4);
	var k9 = K - y * y * j3 + y * y * y * y * j4 - y * y * y * y * y * y * j5;
	var j6 = 1.0 / (c * V);
	var j7 = 1.0 / (c * 6 * V * V * V) * (V / R + 2 * t2);
	var j8 = 1.0 / (c * 120 * V * V * V * V * V) * (5 + 28 * t2 + 24 * t4);
	var j9 = 1.0 / (c * 5040 * V * V * V * V * V * V * V);
	var j9 = j9 * (61 + 662 * t2 + 1320 * t4 + 720 * t6);
	var LL = Lorg + y * j6 - y * y * y * j7 + y * y * y * y * y * j8 - y * y * y * y * y * y * y * j9;
	var KK = k9;
	return new Position(KK, LL);
}

function FNC(K, L)
{
	var V = FNV(K);
	var R = FNR(V, K);
	var H2 = V / R - 1.0;
	var s = Math.sin(K);
	var c = Math.cos(K);
	var t = Math.tan(K);
	var t2 = t * t;
	var c2 = c * c;
	var c4 = c2 * c2;
	var P = L - Lorg;
	var j4 = s * c2 / 3.0 * (1 + 3 * H2 + 2 * H2 * H2);
	var j5 = s * c4 / 15.0 * (2 - t2);
	return ( P * s + P * P * P * j4 + P * P * P * P * P * j5 );
}

function con(E, N)
{
	var K = Phi(N);
	var t = Math.tan(K);
	var t2 = t * t;
	var t4 = t2 * t2;
	var V = FNV(K);
	var R = FNR(V, K);
	var H2 = V / R - 1.0;
	var y = E - Eorg;
	var V = 0.000001 * V;
	var R = 0.000001 * R;
	var y = 0.000001 * y;
	var j3 = t / V;
	var j4 = t / (3 * V * V * V) * (1 + t2 - H2 - 2 * H2 * H2);
	var j5 = t / (15 * V * V * V * V * V) * (2 + 5 * t2 + 3 * t4);
	return ( y * j3 - y * y * y * j4 + y * y * y * y * y * j5 );
}

function FNF(K, L)
{
	var V = FNV(K);
	var R = FNR(V, K);
	var H2 = V / R - 1.0;
	var c = Math.cos(K);
	var t = Math.tan(K);
	var t2 = t * t;
	var c2 = c * c;
	var c4 = c2 * c2;
	var P = L - Lorg;
	var j3 = c2 / 2.0 * (1 + H2);
	var j4 = c4 / 24.0 * (5 - 4 * t2 + 14 * H2 - 28 * t2 * H2);
	return ( F0 * (1 + P * P * j3 + P * P * P * P * j4) );
}

function fac(E, N)
{
	var K = Phi(N);
	var V = FNV(K);
	var R = FNR(V, K);
	var H2 = V / R - 1.0;
	var y = E - Eorg;
	var V = 0.000001 * V;
	var R = 0.000001 * R;
	var y = 0.000001 * y;
	var j3 = 1.0 / (2 * R * V);
	var j4 = (1 + 4 * H2) / (24 * R * R * V * V);
	return ( F0 * (1 + y * y * j3 + y * y * y * y * j4) );
}

function deltaTa(Ea, Na, Eb, Nb)
{
	var K = Phi(0.5 * (Na + Nb));
	var V = FNV(K);
	var R = FNR(V, K);
	var H2 = V / R - 1.0;
	var ya = Ea - Eorg;
	var yb = Eb - Eorg;
	var V = 0.001 * V;
	var R = 0.001 * R;
	var ya = 0.000001 * ya;
	var yb = 0.000001 * yb;
	var j3 = 1.0 / (6 * R * V);
	return ( (2 * ya + yb) * (Na - Nb) * j3 );
}

function deltaTb(Ea, Na, Eb, Nb)
{
	var K = Phi(0.5 * (Na + Nb));
	var V = FNV(K);
	var R = FNR(V, K);
	var H2 = V / R - 1.0;
	var ya = Ea - Eorg;
	var yb = Eb - Eorg;
	var V = 0.001 * V;
	var R = 0.001 * R;
	var ya = 0.000001 * ya;
	var yb = 0.000001 * yb;
	var j3 = 1.0 / (6 * R * V);
	return ( (ya + 2 * yb) * (Nb - Na) * j3 );
}
